@@ -1,5 +1,6 @@
 from abc import ABC, abstractmethod
 from collections import defaultdict
+import re
 
 from sigma.exceptions import SigmaError, SigmaValueError
 from sigma.conversion.deferred import DeferredQueryExpression
@@ -353,6 +354,7 @@ class TextQueryBackend(Backend):
     # The backend generates grouping if required
     precedence : ClassVar[Tuple[ConditionItem, ConditionItem, ConditionItem]] = (ConditionNOT, ConditionAND, ConditionOR)
     group_expression : ClassVar[Optional[str]] = None   # Expression for precedence override grouping as format string with {expr} placeholder
+    parenthesize : bool = False     # Reflect parse tree by putting parenthesis around all expressions - use this for target systems without strict precedence rules.
 
     # Generated query tokens
     token_separator : str = " "     # separator inserted between all boolean operators
@@ -363,18 +365,28 @@ class TextQueryBackend(Backend):
 
     # String output
     ## Fields
+    ### Quoting
     field_quote : ClassVar[Optional[str]] = None                # Character used to quote field characters if field_quote_pattern matches (or not, depending on field_quote_pattern_negation). No field name quoting is done if not set.
     field_quote_pattern : ClassVar[Optional[Pattern]] = None    # Quote field names if this pattern (doesn't) matches, depending on field_quote_pattern_negation. Field name is always quoted if pattern is not set.
     field_quote_pattern_negation : ClassVar[bool] = True        # Negate field_quote_pattern result. Field name is quoted if pattern doesn't matches if set to True (default).
-    # TODO: add field name escaping at least for quote character
+
+    ### Escaping
+    field_escape : ClassVar[Optional[str]] = None               # Character to escape particular parts defined in field_escape_pattern.
+    field_escape_quote : ClassVar[bool] = True                  # Escape quote string defined in field_quote
+    field_escape_pattern : ClassVar[Optional[Pattern]] = None   # All matches of this pattern are prepended with the string contained in field_escape.
 
     ## Values
-    str_quote       : ClassVar[Optional[str]] = None    # string quoting character (added as escaping character)
-    escape_char     : ClassVar[Optional[str]] = None    # Escaping character for special characrers inside string
+    ### String quoting
+    str_quote       : ClassVar[str] = ""                # string quoting character (added as escaping character)
+    str_quote_pattern : ClassVar[Optional[Pattern]] = None      # Quote string values that match (or don't match) this pattern
+    str_quote_pattern_negation : ClassVar[bool] = True  # Negate str_quote_pattern result
+    ### String escaping and filtering
+    escape_char     : ClassVar[Optional[str]] = None    # Escaping character for special characters inside string
     wildcard_multi  : ClassVar[Optional[str]] = None    # Character used as multi-character wildcard
     wildcard_single : ClassVar[Optional[str]] = None    # Character used as single-character wildcard
     add_escaped     : ClassVar[str] = ""                # Characters quoted in addition to wildcards and string quote
     filter_chars    : ClassVar[str] = ""                # Characters filtered
+    ### Booleans
     bool_values     : ClassVar[Dict[bool, Optional[str]]] = {   # Values to which boolean values are mapped.
         True: None,
         False: None,
@@ -391,10 +403,9 @@ class TextQueryBackend(Backend):
     re_escape_char : ClassVar[Optional[str]] = None     # Character used for escaping in regular expressions
     re_escape : ClassVar[Tuple[str]] = ()               # List of strings that are escaped
 
-    # cidr expressions
-    cidr_wildcard : ClassVar[Optional[str]] = None    # Character used as single wildcard
-    cidr_expression : ClassVar[Optional[str]] = None    # CIDR expression query as format string with placeholders {field} = {value}
-    cidr_in_list_expression : ClassVar[Optional[str]] = None    # CIDR expression query as format string with placeholders {field} = in({list})
+    # CIDR expressions: define CIDR matching if backend has native support. Else pySigma expands
+    # CIDR values into string wildcard matches.
+    cidr_expression : ClassVar[Optional[str]] = None    # CIDR expression query as format string with placeholders {field}, {value} (the whole CIDR value), {network} (network part only), {prefixlen} (length of network mask prefix) and {netmask} (CIDR network mask only)
 
     # Numeric comparison operators
     compare_op_expression : ClassVar[Optional[str]] = None      # Compare operation query as format string with placeholders {field}, {operator} and {value}
@@ -425,6 +436,9 @@ class TextQueryBackend(Backend):
         enclosing condition item (outer) is lower than the contained (inner) condition item.
         In this case, no additional grouping is required.
         """
+        if self.parenthesize and not isinstance(inner, ( ConditionFieldEqualsValueExpression, ConditionValueExpression )):       # if parenthesize is set, parenthesis are generally put around everything.
+            return False
+
         outer_class = outer.__class__
         # Special case: Conditions containing a SigmaExpansion value convert into OR conditions and therefore the precedence has to be handled the same way.
         if isinstance(inner, ( ConditionFieldEqualsValueExpression, ConditionValueExpression )) and isinstance(inner.value, SigmaExpansion):
@@ -466,10 +480,10 @@ class TextQueryBackend(Backend):
     def convert_condition_as_in_expression(self, cond : Union[ConditionOR, ConditionAND], state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of field in value list conditions."""
         return self.field_in_list_expression.format(
-            field=self.quote_field(cond.args[0].field),       # The assumption that the field is the same for all argument is valid because this is checked before
+            field=self.escape_and_quote_field(cond.args[0].field),       # The assumption that the field is the same for all argument is valid because this is checked before
             op=self.or_in_operator if isinstance(cond, ConditionOR) else self.and_in_operator,
             list=self.list_separator.join([
-                self.str_quote + self.convert_value_str(arg.value, state) + self.str_quote
+                self.convert_value_str(arg.value, state)
                 if isinstance(arg.value, SigmaString)   # string escaping and qouting
                 else str(arg.value)       # value is number
                 for arg in cond.args
@@ -511,30 +525,91 @@ class TextQueryBackend(Backend):
         except TypeError:       # pragma: no cover
             raise NotImplementedError("Operator 'not' not supported by the backend")
 
-    def quote_field(self, field_name : str) -> str:
-        """Quote field name with field_quote if field_quote_pattern (doesn't) matches, depending on
-        field_quote_pattern_negation."""
-        if self.field_quote is not None:
-            if self.field_quote_pattern is not None:
-                quote = bool(self.field_quote_pattern.match(field_name))
-                if self.field_quote_pattern_negation:
+    def escape_and_quote_field(self, field_name : str) -> str:
+        """
+        Escape field name by prepending pattern matches of field_escape_pattern with field_escape
+        string. If field_escape_quote is set to True (default) and field escaping string is defined
+        in field_escape, all instances of the field quoting character are escaped before quoting.
+
+        Quote field name with field_quote if field_quote_pattern (doesn't) matches the original
+        (unescaped) field name. If field_quote_pattern_negation is set to True (default) the pattern matching
+        result is negated, which is the default behavior. In this case the field name is quoted if
+        the pattern doesn't matches.
+        """
+        if self.field_escape is not None:               # field name escaping
+            if self.field_escape_pattern is not None:   # Match all occurrences of field_escpae_pattern if defined and initialize match position set with result.
+                match_positions = {
+                    match.start()
+                    for match in self.field_escape_pattern.finditer(field_name)
+                }
+            else:
+                match_positions = set()
+
+            if self.field_escape_quote and self.field_quote is not None:         # Add positions of quote string to match position set
+                re_quote = re.compile(re.escape(self.field_quote))
+                match_positions.update((
+                    match.start()
+                    for match in re_quote.finditer(field_name)
+                ))
+
+            if len(match_positions) > 0:    # found matches, escape them
+                r = [0] + list(sorted(match_positions)) + [len(field_name)]
+                escaped_field_name = ""
+                for i in range(len(r) - 1):    # TODO: from Python 3.10 this can be replaced with itertools.pairwise(), but for now we keep support for Python <3.10
+                    if i == 0:          # The first range is passed to the result without escaping
+                        escaped_field_name += field_name[r[i]:r[i + 1]]
+                    else:               # Subsequent ranges are positions of matches and therefore are prepended with field_escape
+                        escaped_field_name += self.field_escape + field_name[r[i]:r[i + 1]]
+            else:                       # no matches, just pass original field name without escaping
+                escaped_field_name = field_name
+        else:
+            escaped_field_name = field_name
+
+        if self.field_quote is not None:                # Field quoting
+            if self.field_quote_pattern is not None:    # Match field quote pattern...
+                quote = bool(self.field_quote_pattern.match(escaped_field_name))
+                if self.field_quote_pattern_negation:   # ...negate result of matching, if requested...
                     quote = not quote
             else:
                 quote = True
 
-            if quote:
-                return self.field_quote + field_name + self.field_quote
-        return field_name
+            if quote:                                   #  ...and quote if pattern (doesn't) matches
+                return self.field_quote + escaped_field_name + self.field_quote
+        return escaped_field_name
 
-    def convert_value_str(self, s : SigmaString, state : ConversionState) -> Union[str, DeferredQueryExpression]:
+    def decide_string_quoting(self, s : SigmaString) -> bool:
+        """
+        Decide if string is quoted based on the pattern in the class attribute str_quote_pattern. If
+        this matches (or not matches if str_quote_pattern_negation is set to True), the string is quoted.
+        """
+        if self.str_quote == "":    # No quoting if quoting string is empty.
+            return False
+
+        if self.str_quote_pattern is None:      # Always quote if pattern is not set.
+            return True
+        else:
+            match = bool(self.str_quote_pattern.match(str(s)))
+            if self.str_quote_pattern_negation:
+                match = not match
+            return match
+
+    def quote_string(self, s : str) -> str:
+        """Put quotes around string."""
+        return self.str_quote + s + self.str_quote
+
+    def convert_value_str(self, s : SigmaString, state : ConversionState) -> str:
         """Convert a SigmaString into a plain string which can be used in query."""
-        return s.convert(
+        converted = s.convert(
             self.escape_char,
             self.wildcard_multi,
             self.wildcard_single,
             self.str_quote + self.add_escaped,
             self.filter_chars,
         )
+        if self.decide_string_quoting(s):
+            return self.quote_string(converted)
+        else:
+            return converted
 
     def convert_condition_field_eq_val_str(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of field = string value expressions"""
@@ -568,23 +643,23 @@ class TextQueryBackend(Backend):
                 expr = self.wildcard_match_expression
                 value = cond.value
             else:
-                expr =  "{field}" + self.eq_token + self.str_quote + "{value}" + self.str_quote # [New]
+                expr =  "{field}" + self.eq_token + "{value}"
                 value = cond.value
-            return expr.format(field=self.quote_field(cond.field), value=self.convert_value_str(value, state))
+            return expr.format(field=self.escape_and_quote_field(cond.field), value=self.convert_value_str(value, state))
         except TypeError:       # pragma: no cover
             raise NotImplementedError("Field equals string value expressions with strings are not supported by the backend.")
 
     def convert_condition_field_eq_val_num(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of field = number value expressions"""
         try:
-            return self.quote_field(cond.field) + self.eq_token + str(cond.value)
+            return self.escape_and_quote_field(cond.field) + self.eq_token + str(cond.value)
         except TypeError:       # pragma: no cover
             raise NotImplementedError("Field equals numeric value expressions are not supported by the backend.")
 
     def convert_condition_field_eq_val_bool(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of field = bool value expressions"""
         try:
-            return self.quote_field(cond.field) + self.eq_token + self.bool_values[cond.value.boolean]
+            return self.escape_and_quote_field(cond.field) + self.eq_token + self.bool_values[cond.value.boolean]
         except TypeError:       # pragma: no cover
             raise NotImplementedError("Field equals numeric value expressions are not supported by the backend.")
 
@@ -592,66 +667,48 @@ class TextQueryBackend(Backend):
         """Convert regular expression into string representation used in query."""
         return r.escape(self.re_escape, self.re_escape_char)
 
-    def convert_value_cidr(self, ip : SigmaCIDRExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
-        """Convert regular expression into string representation used in query."""
-        return ip.convert(join_expr=self.or_token,wildcard=self.cidr_wildcard)
-
     def convert_condition_field_eq_val_re(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of field matches regular expression value expressions."""
         return self.re_expression.format(
-            field=self.quote_field(cond.field),
+            field=self.escape_and_quote_field(cond.field),
             regex=self.convert_value_re(cond.value, state),
         )
 
     def convert_condition_field_eq_val_re_contains(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of value-only regular expressions."""
         return self.re_expression.format(
-            field=self.quote_field(cond.field),
+            field=self.escape_and_quote_field(cond.field),
             regex=self.convert_value_re(cond.value, state),
         )
 
     def convert_condition_field_eq_val_cidr(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
-        """Conversion of field matches regular expression value expressions."""
-        convert_str = self.convert_value_cidr(cond.value, state)
-        if self.or_token in convert_str:
-            list_ip = convert_str.split(self.or_token)
-            if self.cidr_wildcard == None:
-                    return self.cidr_in_list_expression.format(
-                        field=self.quote_field(cond.field),
-                        list=self.list_separator.join([str(v) for v in list_ip])
-                    )
-            else:
-                return self.cidr_in_list_expression.format(
-                    field=self.quote_field(cond.field),
-                    list=self.list_separator.join([ self.str_quote + str(v) + self.str_quote for v in list_ip])
-                )
-        else:
-            if self.cidr_wildcard == None:
-                return self.cidr_expression.format(
-                    field=self.quote_field(cond.field),
-                    value=convert_str,
-                )
-            else:
-                return self.cidr_expression.format(
-                    field=self.quote_field(cond.field),
-                    value=self.str_quote + convert_str + self.str_quote,
-                )
+        """Conversion of field matches CIDR value expressions."""
+        cidr : SigmaCIDRExpression = cond.value
+        if self.cidr_expression is not None:        # native CIDR support from backend with expression templates.
+            return self.cidr_expression.format(field=cond.field, value=str(cidr.network), network=cidr.network.network_address, prefixlen=cidr.network.prefixlen, netmask=cidr.network.netmask)
+        else:                                       # No native CIDR support: expand into string wildcard matches on prefixes.
+            expanded = cidr.expand(self.wildcard_multi)
+            expanded_cond = ConditionOR([
+                ConditionFieldEqualsValueExpression(cond.field, SigmaString(network))
+                for network in expanded
+            ], cond.source)
+            return self.convert_condition(expanded_cond, state)
 
     def convert_condition_field_compare_op_val(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of numeric comparison operations into queries."""
         return self.compare_op_expression.format(
-            field=self.quote_field(cond.field),
+            field=self.escape_and_quote_field(cond.field),
             operator=self.compare_operators[cond.value.op],
             value=cond.value.number,
         )
 
     def convert_condition_field_eq_val_null(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of field is null expression value expressions"""
-        return self.field_null_expression.format(field=self.quote_field(cond.field))
+        return self.field_null_expression.format(field=self.escape_and_quote_field(cond.field))
 
     def convert_condition_field_eq_query_expr(self, cond : ConditionFieldEqualsValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of field is null expression value expressions"""
-        return cond.value.finalize(field=self.quote_field(cond.field))
+        return cond.value.finalize(field=self.escape_and_quote_field(cond.field))
 
     def convert_condition_val_str(self, cond : ConditionValueExpression, state : ConversionState) -> Union[str, DeferredQueryExpression]:
         """Conversion of value-only strings."""
