@@ -11,7 +11,7 @@ from sigma.modifiers import SigmaModifier, modifier_mapping, reverse_modifier_ma
 from sigma.conditions import SigmaCondition, ConditionAND, ConditionOR, ConditionFieldEqualsValueExpression, ConditionValueExpression, ParentChainMixin
 from sigma.processing.tracking import ProcessingItemTrackingMixin
 import sigma.exceptions as sigma_exceptions
-from sigma.exceptions import SigmaRuleLocation
+from sigma.exceptions import SigmaRuleLocation, SigmaValueError
 
 class EnumLowercaseStringMixin:
     def __str__(self) -> str:
@@ -40,7 +40,10 @@ class SigmaRuleTag:
     @classmethod
     def from_str(cls, tag : str, source : Optional[SigmaRuleLocation] = None) -> "SigmaRuleTag":
         """Build SigmaRuleTag class from plain text tag string."""
-        ns, n = tag.split(".", maxsplit=1)
+        try:
+            ns, n = tag.split(".", maxsplit=1)
+        except ValueError as e:
+            raise SigmaValueError("Sigma tag must start with namespace separated with dot from remaining tag.")
         return cls(ns, n)
 
     def __str__(self) -> str:
@@ -295,19 +298,20 @@ class SigmaDetection(ParentChainMixin):
     3. a list of plain values or mappings defined and matched as in 1 where at least one of the items should appear in matched events.
     """
     detection_items : List[Union[SigmaDetectionItem, "SigmaDetection"]]
-    item_linking : Union[Type[ConditionAND], Type[ConditionOR]] = field(init=False)
     source : Optional[SigmaRuleLocation] = field(default=None, compare=False)
+    item_linking : Union[Type[ConditionAND], Type[ConditionOR]] = field(default=None)
 
     def __post_init__(self):
         """Check detection validity."""
         if len(self.detection_items) == 0:
             raise sigma_exceptions.SigmaDetectionError("Detection is empty", source=self.source)
 
-        type_set = { type(item) for item in self.detection_items }
-        if SigmaDetectionItem in type_set:
-            self.item_linking = ConditionAND
-        else:
-            self.item_linking = ConditionOR
+        if self.item_linking is None:
+            type_set = { type(item) for item in self.detection_items }
+            if SigmaDetectionItem in type_set:
+                self.item_linking = ConditionAND
+            else:
+                self.item_linking = ConditionOR
 
     @classmethod
     def from_definition(cls, definition : Union[Mapping, Sequence, str, int], source : Optional[SigmaRuleLocation] = None) -> "SigmaDetection":
@@ -413,7 +417,7 @@ class SigmaDetection(ParentChainMixin):
                                 del merged[k]
 
                 return {
-                    k: (v[0] if len(v) == 1 else v)
+                    k: (v[0] if isinstance(v, list) and len(v) == 1 else v)
                     for k, v in merged.items()
                 }
             else:       # only lists and plain values, merge them into one list
